Package: ecolMod
Version: 1.2.6.4
Date: 2022-11-13
Title: "A Practical Guide to Ecological Modelling - Using R as a
        Simulation Platform"
Author: Karline Soetaert <karline.soetaert@nioz.nl>, Peter MJ Herman <peter.herman@nioz.nl>
Maintainer: Karline Soetaert <karline.soetaert@nioz.nl>
Depends: R (>= 2.01), rootSolve, shape, diagram, deSolve
Imports: stats, graphics, grDevices
Suggests: maps, seacarb, scatterplot3d, deldir
Description: Figures, data sets and examples from the book "A practical guide to ecological modelling - using R as a simulation platform" by Karline Soetaert and Peter MJ Herman (2009). Springer.
  All figures from chapter x can be generated by "demo(chapx)", where x = 1 to 11. 
  The R-scripts of the model examples discussed in the book are in subdirectory "examples", ordered per chapter.
  Solutions to model projects are in the same subdirectories.
License: GPL (>= 2)
LazyData: yes
NeedsCompilation: yes
Packaged: 2022-11-13 16:27:33 UTC; karlines
Repository: CRAN
Date/Publication: 2022-11-15 19:50:05 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 18:31:35 UTC; unix
Archs: ecolMod.so.dSYM
